@@ -1,15 +1,19 @@
+import logging
+import time
 from functools import partial
 from unittest import TestCase
 from uuid import uuid4
 
 from pydantic.types import conint
 
-from openmodule.models import OpenModuleModel
+from openmodule.config import settings
+from openmodule.models.base import OpenModuleModel
 from openmodule.rpc import server
+from openmodule.rpc.client import RPCClient
 from openmodule.rpc.common import channel_to_response_topic, channel_to_request_topic
 from openmodule.rpc.server import RPCServer, RPCRequest
+from openmodule_test.core import OpenModuleCoreTestMixin
 from openmodule_test.rpc import RPCServerTestMixin
-from openmodule_test.zeromq import ZMQTestMixin
 
 
 def _fake_request(gate=..., direction=...):
@@ -81,7 +85,7 @@ class TestRPCResponse2(OpenModuleModel):
     some_payload: str
 
 
-class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
+class RPCServerTestCase(RPCServerTestMixin):
     rpc_channels = ["channel", "channel2"]
 
     server: RPCServer
@@ -98,6 +102,7 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
         super().tearDown()
 
     def set_called(self, request, message: RPCRequest, value=True):
+        """test rpc handler"""
         self.called_types[message.type] = value
 
     def set_called_value(self, value):
@@ -167,6 +172,38 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
             self.rpc("channel", "test-type2", {})
         self.assertNotIn("test-type2", self.called_types)
 
+    def test_filter_channel_type(self):
+        def f(*args, **kwargs):
+            return False
+
+        def check_ok():
+            response = self.rpc("channel", "test-type", {})
+            self.assertRPCSuccess(response)
+            self.assertTrue(self.called_types.get("test-type"))
+            self.called_types.clear()
+
+        def check_not_called():
+            with self.assertRaises(TimeoutError):
+                response = self.rpc("channel", "test-type", {})
+            self.assertEqual(None, self.called_types.get("test-type"))
+            self.called_types.clear()
+
+        self.server.register_handler("channel", "test-type", TestRPCRequest, TestRPCResponse, self.set_called)
+        self.wait_for_rpc_server(self.server)
+
+        self.server.add_filter(f, "abc")
+        check_ok()
+
+        self.server.add_filter(f, "channel", "abc")
+        check_ok()
+
+        self.server.add_filter(f, "channel")
+        check_not_called()
+
+        self.server.filters.clear()
+        self.server.add_filter(f, "channel", "test-type")
+        check_not_called()
+
     def test_can_only_register_once(self):
         self.server.register_handler("channel", "test-type", TestRPCRequest, TestRPCResponse, self.set_called)
         self.wait_for_rpc_server(self.server)
@@ -176,6 +213,7 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
 
     def test_exception_in_handler(self):
         def handler(*_, **__):
+            """ test rpc handler"""
             raise Exception("Error123")
 
         self.server.register_handler("channel", "test-type", TestRPCRequest, TestRPCResponse, handler)
@@ -187,6 +225,7 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
 
     def test_handler_returns_none(self):
         def handler(*_, **__):
+            """ test rpc handler"""
             return None
 
         self.server.register_handler("channel", "test-type", TestRPCRequest, TestRPCResponse, handler)
@@ -197,6 +236,7 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
 
     def test_handler_returns_dict(self):
         def handler(*_, **__):
+            """ test rpc handler"""
             return {}
 
         self.server.register_handler("channel", "test-type", TestRPCRequest, TestRPCResponse, handler)
@@ -207,6 +247,7 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
 
     def test_handler_returns_model(self):
         def handler(*_, **__):
+            """ test rpc handler"""
             return TestRPCResponse()
 
         self.server.register_handler("channel", "test-type", TestRPCRequest, TestRPCResponse, handler)
@@ -217,9 +258,11 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
 
     def test_handler_returns_wrong_dict(self):
         def bad_handler(*_, **__):
+            """ test rpc handler"""
             return {}
 
         def good_handler(*_, **__):
+            """ test rpc handler"""
             return {"some_payload": "test"}
 
         self.server.register_handler("channel", "test-type1", TestRPCRequest, TestRPCResponse2, bad_handler)
@@ -234,9 +277,11 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
 
     def test_handler_returns_wrong_response_type(self):
         def bad_handler(*_, **__):
+            """ test rpc handler"""
             return TestRPCResponse()
 
         def good_handler(*_, **__):
+            """ test rpc handler"""
             return TestRPCResponse2(some_payload="test")
 
         self.server.register_handler("channel", "test-type1", TestRPCRequest, TestRPCResponse2, bad_handler)
@@ -255,6 +300,7 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
         """
 
         def too_much_handler(*_, **__):
+            """ test rpc handler"""
             return {"some_payload": "test", "nobody_wants_you": ":("}
 
         self.server.register_handler("channel", "test-type1", TestRPCRequest, TestRPCResponse2, too_much_handler)
@@ -269,6 +315,7 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
             max_int: conint(gt=0, lt=10)
 
         def handler(*_, **__):
+            """ test rpc handler"""
             pass
 
         self.server.register_handler("channel", "test-type", SomeValidator, TestRPCResponse, handler)
@@ -281,6 +328,7 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
 
     def test_no_handler_found(self):
         def handler(*_, **__):
+            """ test rpc handler"""
             return TestRPCResponse()
 
         self.server.register_handler("channel", "test-type", TestRPCRequest, TestRPCResponse, handler)
@@ -297,3 +345,70 @@ class RPCServerTestCase(RPCServerTestMixin, ZMQTestMixin, TestCase):
 
         with self.assertRaises(TimeoutError):  # wrong everything
             self.rpc("channel2", "test-type-doesnotexist", {})
+
+    def test_correct_resource(self):
+        def handler1(*_, **__):
+            """ test rpc handler"""
+            logging.info("handler 1")
+            return TestRPCResponse()
+
+        self.server.register_handler("channel", "test-type", TestRPCRequest, TestRPCResponse, handler1)
+        self.wait_for_rpc_server(self.server)
+
+        with self.assertLogs() as cm:
+            response = self.rpc("channel", "test-type", {}, resource=None)
+        self.assertRPCSuccess(response)
+        self.assertIn("handler 1", str(cm.output))
+
+        with self.assertLogs() as cm:
+            response = self.rpc("channel", "test-type", {}, resource=settings.RESOURCE)
+        self.assertRPCSuccess(response)
+        self.assertIn("handler 1", str(cm.output))
+
+        with self.assertRaises(TimeoutError):
+            self.rpc("channel", "test-type", {}, resource="test-2")
+
+
+class RpcClientTest(RPCServerTestMixin, OpenModuleCoreTestMixin):
+    rpc_channels = ["channel"]
+
+    @staticmethod
+    def handler1(*_, **__):
+        """test handler"""
+        logging.info("handler 1")
+        return TestRPCResponse2(some_payload="abc")
+
+    @staticmethod
+    def timeout_handler(*_, **__):
+        """test handler"""
+        logging.info("timout_handler")
+        time.sleep(3)
+        return TestRPCResponse2(some_payload="abc")
+
+    def setUp(self):
+        super().setUp()
+        self.server = RPCServer(config=self.zmq_config(), context=self.zmq_context(), filter_resource=False)
+        self.server.register_handler("channel", "test", TestRPCRequest, TestRPCResponse2, self.handler1)
+        self.server.register_handler("channel", "timeout", TestRPCRequest, TestRPCResponse2, self.timeout_handler)
+        self.server.run_as_thread()
+        self.wait_for_rpc_server(self.server)
+
+        self.client = RPCClient(self.core, channels=[b"channel"])
+        self.wait_for_dispatcher(self.core.messages)
+
+    def test_blocking(self):
+        result = self.client.rpc(b"channel", "test", {}, timeout=1)
+        self.assertEqual("ok", result.status)
+
+        with self.assertRaises(TimeoutError):
+            result = self.client.rpc(b"channel", "timeout", {}, timeout=1)
+
+    def test_non_blocking(self):
+        result = self.client.rpc(b"channel", "test", {}, timeout=1, blocking=False)
+        self.assertEqual("ok", result.status)
+        self.assertEqual(None, result.response)
+        time.sleep(1)
+        result = self.client.check_result(result.rpc_id)
+        self.assertIsNotNone(result.response)
+        self.assertEqual("ok", result.status)
+        self.assertIn("some_payload", result.response)
